@@ -1,17 +1,49 @@
 from __future__ import annotations
 
 import argparse
+import logging
+import logging.config
 import os
 import sys
 from abc import ABC, abstractmethod
 from pathlib import Path
 
+from ruamel.yaml import YAML
+
 from .errors import TedsError
-from .generate import generate_from, generate_from_source_config, parse_generate_config
+from .generate import generate_from_source_config, parse_generate_config
 from .validate import validate_file
 from .version import get_version, recommended_minor_str, supported_spec_range_str
 
 
+def setup_logging():
+    """Setup logging from YAML config with environment variable override."""
+    # Find the logging config file
+    config_path = Path(__file__).parent.parent / "logging.yaml"
+    
+    if config_path.exists():
+        yaml = YAML(typ='safe', pure=True)
+        with open(config_path, 'r') as f:
+            config = yaml.load(f)
+        
+        # Override log level from environment variable if set
+        env_level = os.getenv('LOGLEVEL', '').upper()
+        if env_level in ['DEBUG', 'INFO', 'WARNING', 'ERROR', 'CRITICAL']:
+            # Set the teds_core logger level from environment
+            if 'loggers' in config and 'teds_core' in config['loggers']:
+                config['loggers']['teds_core']['level'] = env_level
+        
+        logging.config.dictConfig(config)
+    else:
+        # Fallback to basic config
+        level = os.getenv('LOGLEVEL', 'INFO').upper()
+        logging.basicConfig(
+            level=getattr(logging, level, logging.INFO),
+            format='%(name)s:%(levelname)s: %(message)s',
+            stream=sys.stderr
+        )
+
+
 class Command(ABC):
     """Abstract base class for CLI commands."""
 
@@ -154,52 +186,14 @@ class GenerateCommand(Command):
             for mapping_str in args.mapping:
                 config = parse_generate_config(mapping_str)
 
-                if isinstance(config, dict):
-                    # Source-centric YAML object format - use current working directory
-                    # Note: This assumes relative paths in config are relative to cwd
-                    base_dir = Path.cwd()
-                    # Status messages will be handled inside generate_from_source_config
-                    generate_from_source_config(config, base_dir)
-                else:
-                    # Backward compatibility: JSON Pointer string
-                    # Use the old generate_from system for children expansion
-                    ref_str, target = _split_ref(mapping_str)
+                # All inputs are now normalized to dict format by parse_generate_config
+                if not isinstance(config, dict):
+                    raise TedsError(f"Unexpected configuration format: {type(config)}")
 
-                    # Use the old system with children expansion
-                    if target:
-                        # Parse ref to get components for template expansion
-                        file_part, pointer = _parse_ref(ref_str)
-                        # Expand templates in target path
-                        expanded_target = _expand_target_template(
-                            target, file_part, pointer
-                        )
-                        target_path = Path(expanded_target)
-
-                        # If target ends with /, treat as directory and use default filename
-                        if target.endswith("/"):
-                            base = Path(file_part).stem
-                            default_name = _default_filename(base, pointer)
-                            target_path = target_path / default_name
-
-                        abs_ref_str = (
-                            ref_str  # Use original ref when target is specified
-                        )
-                    else:
-                        file_part, pointer = _parse_ref(ref_str)
-                        base = Path(file_part).stem
-                        # Use absolute path to avoid double directory resolution
-                        if Path(file_part).is_absolute():
-                            schema_dir = Path(file_part).parent
-                            abs_ref_str = ref_str
-                        else:
-                            schema_dir = Path.cwd() / Path(file_part).parent
-                            # Create relative ref from testspec location to schema
-                            schema_filename = Path(file_part).name
-                            abs_ref_str = f"{schema_filename}#{pointer}"
-                        target_path = schema_dir / _default_filename(base, pointer)
-
-                    print(f"Generating {target_path}", file=sys.stderr)
-                    generate_from(abs_ref_str, target_path)
+                # Use unified processing pipeline
+                base_dir = Path.cwd()
+                # Status messages will be handled inside generate_from_source_config
+                generate_from_source_config(config, base_dir)
         except TedsError as e:
             print(str(e), file=sys.stderr)
             return 2
@@ -251,57 +245,6 @@ def _sanitize(s: str) -> str:
     return "+".join(esc)
 
 
-def _split_ref(mapping: str) -> tuple[str, str | None]:
-    if "=" in mapping:
-        left, right = mapping.split("=", 1)
-        return left.strip(), right.strip()
-    return mapping.strip(), None
-
-
-def _parse_ref(ref: str) -> tuple[str, str]:
-    file_part, sep, frag = ref.partition("#")
-    if not sep:
-        return file_part, "/"
-    return file_part, frag or "/"
-
-
-def _tokens_for_mapping(file_part: str, pointer: str, out_index: int) -> dict:
-    from urllib.parse import quote
-
-    p = Path(file_part)
-    file_name = p.name or "schema.yaml"
-    base = p.stem or "schema"
-    ext = p.suffix.lstrip(".") if p.suffix else "yaml"
-    dir_part = str(p.parent) if str(p.parent) not in (".", "") else ""
-
-    pointer_raw = pointer.lstrip("/")
-    pointer_token = _sanitize(pointer_raw) if pointer_raw else "root"
-    pointer_strict = quote(pointer_raw, safe="") if pointer_raw else "root"
-
-    return {
-        "file": file_name,
-        "base": base,
-        "ext": ext,
-        "dir": dir_part,
-        "pointer": pointer_token,
-        "pointer_raw": pointer_raw,
-        "pointer_strict": pointer_strict,
-        "index": out_index,
-    }
-
-
-def _default_filename(base: str, pointer: str, exact_ref: bool = False) -> str:
-    if exact_ref:
-        # For exact references, always use base filename
-        return f"{base}.tests.yaml"
-
-    pointer_raw = pointer.lstrip("/")
-    if not pointer_raw:
-        return f"{base}.tests.yaml"
-    # Always use base filename for consistency with JSON-Path behavior
-    return f"{base}.tests.yaml"
-
-
 def _expand_target_template(target: str, file_part: str, pointer: str) -> str:
     """Expand template variables in target path."""
     import urllib.parse
@@ -336,64 +279,6 @@ def _expand_target_template(target: str, file_part: str, pointer: str) -> str:
     return expanded
 
 
-def _plan_pairs(mappings: list[str]) -> list[tuple[str, Path]]:
-    pairs: list[tuple[str, Path]] = []
-
-    # Check if this is a single exact reference (for exact node behavior)
-    is_single_exact = (
-        len(mappings) == 1 and "#" in mappings[0] and "*" not in mappings[0]
-    )
-
-    for i, m in enumerate(mappings, start=1):
-        ref_str, target = _split_ref(m)
-        file_part, pointer = _parse_ref(ref_str)
-        toks = _tokens_for_mapping(file_part, pointer, i)
-
-        schema_dir = Path(file_part).resolve().parent
-
-        if target is None:
-            out_path = schema_dir / _default_filename(
-                toks["base"], pointer, exact_ref=is_single_exact
-            )
-        else:
-            target_fmt = target.format(**toks) if "{" in target else target
-            base_target = Path(target_fmt)
-            path = (
-                base_target if base_target.is_absolute() else (schema_dir / base_target)
-            )
-            if target_fmt.endswith(os.sep) or path.is_dir():
-                out_path = path / _default_filename(
-                    toks["base"], pointer, exact_ref=is_single_exact
-                )
-            else:
-                out_path = path
-
-        # Build a relative file path for the ref based on the output directory
-        file_path = Path(file_part)
-        try:
-            file_rel = os.path.relpath(file_path.resolve(), out_path.parent.resolve())
-        except ValueError:
-            # Different drives on Windows, fall back to absolute
-            file_rel = str(file_path.resolve())
-
-        ref_out = (
-            f"{file_rel}#/{pointer.lstrip('/')}"
-            if not file_rel.endswith(f"#{pointer}")
-            else f"{file_rel}#{pointer}"
-        )
-        pairs.append((ref_out, out_path))
-    outs_abs = [p.resolve() for _, p in pairs]
-    seen = {}
-    for idx, p in enumerate(outs_abs):
-        if p in seen:
-            other = seen[p]
-            raise TedsError(
-                f"Output collision: mappings #{other+1} and #{idx+1} both target {p}"
-            )
-        seen[p] = idx
-    return [(ref, p) for (ref, _), p in zip(pairs, outs_abs, strict=False)]
-
-
 def _build_parser() -> argparse.ArgumentParser:
     ap = argparse.ArgumentParser(
         prog="teds",
@@ -444,6 +329,7 @@ def _build_parser() -> argparse.ArgumentParser:
     )
     p_verify.add_argument("spec", nargs="+", help="YAML testspec file(s)")
     p_verify.add_argument(
+        "-l",
         "--output-level",
         choices=["all", "warning", "error"],
         default="warning",
@@ -478,16 +364,15 @@ def _build_parser() -> argparse.ArgumentParser:
             "  {base}  schema filename without extension\n"
             "  {ext}   schema extension without dot\n"
             "  {dir}   schema file directory (as given)\n"
-            "  {pointer}      JSON Pointer (no leading '/'), sanitized for filenames\n"
+            "  {pointer}      JSON Pointer (no leading '/'), RFC 6901 escaped for filenames\n"
             "  {pointer_raw}  JSON Pointer without leading '/' (slashes preserved)\n"
-            "  {pointer_strict} JSON Pointer without leading '/', percent-encoded (safe, reversible)\n"
-            "  {index}        1-based index of this mapping\n\n"
+            "  {pointer_strict} JSON Pointer without leading '/', percent-encoded (safe, reversible)\n\n"
             "Default filename (when TARGET is omitted or a directory):\n"
             "  - Pointer '#/': {base}.tests.yaml\n"
-            "  - Any other pointer: {base}.{pointer}.tests.yaml (pointer without leading '/', sanitized)\n\n"
+            "  - Any other pointer: {base}.{pointer}.tests.yaml (pointer without leading '/', RFC 6901 escaped)\n\n"
             "Examples:\n"
             "  teds generate demo/schema.yaml#/components/schemas\n"
-            "    → writes demo/schema.components+schemas.tests.yaml\n"
+            "    → writes demo/schema.components~1schemas.tests.yaml\n"
         ),
         formatter_class=argparse.RawTextHelpFormatter,
     )
@@ -500,6 +385,7 @@ def _build_parser() -> argparse.ArgumentParser:
 
 def main() -> None:
     """Main CLI entry point using Command pattern."""
+    setup_logging()  # Initialize logging first
     argv = sys.argv[1:]
     registry = CommandRegistry()
 
