@@ -1,7 +1,7 @@
 from __future__ import annotations
 
 import os
-import re
+import logging
 import sys
 from pathlib import Path
 from typing import Any
@@ -11,9 +11,28 @@ from ruamel.yaml.comments import CommentedMap
 
 from .errors import TedsError
 from .refs import collect_examples, join_fragment, resolve_schema_node
+from .utils import expand_target_template, json_path_to_json_pointer
 from .version import RECOMMENDED_TESTSPEC_VERSION
 from .yamlio import yaml_dumper, yaml_loader
 
+# Configure logger for this module
+logger = logging.getLogger(__name__)
+
+
+def _write_yaml(file_path: Path, doc: dict) -> None:
+    """Write YAML document to file, creating parent directories as needed."""
+    try:
+        # Ensure parent directory exists (user-friendly feature)
+        file_path.parent.mkdir(parents=True, exist_ok=True)
+        with file_path.open("w", encoding="utf-8") as fh:
+            yaml_dumper.dump(doc, fh)
+        logger.debug(f"Successfully wrote YAML file {file_path}")
+    except Exception as e:
+        logger.debug(f"Failed to write YAML file {file_path}: {type(e).__name__}: {e}")
+        raise TedsError(
+            f"Failed to write testspec: {file_path}\n  error: {type(e).__name__}: {e}"
+        ) from e
+
 
 def _ensure_group(group: Any) -> dict[str, Any]:
     group = {} if not isinstance(group, dict) else dict(group)
@@ -22,8 +41,17 @@ def _ensure_group(group: Any) -> dict[str, Any]:
     return group
 
 
-def generate_exact_node(node_ref: str, testspec_path: Path) -> None:
+def _create_empty_test_file(testspec_path: Path) -> None:
+    """Create an empty test file with basic structure for visibility in batch mode."""
+    doc = {"version": RECOMMENDED_TESTSPEC_VERSION, "tests": {}}
+
+    # Write the empty test file
+    _write_yaml(testspec_path, doc)
+
+
+def generate_exact_node(node_ref: str, testspec_path: Path, schema_base_dir: Path = None) -> None:
     """Generate a test for the exact referenced node without expanding children."""
+    logger.debug(f"generate_exact_node: Starting with node_ref='{node_ref}', testspec_path={testspec_path}, schema_base_dir={schema_base_dir}")
     if testspec_path.exists():
         try:
             doc = yaml_loader.load(testspec_path.read_text(encoding="utf-8")) or {}
@@ -43,11 +71,18 @@ def generate_exact_node(node_ref: str, testspec_path: Path) -> None:
         tests = {}
         doc["tests"] = tests
 
-    base_dir = testspec_path.parent
+    # Use provided schema_base_dir or fall back to testspec directory
+    if schema_base_dir is not None:
+        base_dir = schema_base_dir
+        logger.debug(f"generate_exact_node: using provided schema_base_dir={base_dir}")
+    else:
+        base_dir = testspec_path.parent
+        logger.debug(f"generate_exact_node: using testspec parent as base_dir={base_dir}")
 
     # Create test only for the exact referenced node (no children expansion)
     group = _ensure_group(tests.get(node_ref))
     tests[node_ref] = group
+    logger.debug(f"generate_exact_node: Created test group for node_ref='{node_ref}'")
 
     # Add examples if available
     ex_list = collect_examples(base_dir, node_ref)
@@ -64,13 +99,8 @@ def generate_exact_node(node_ref: str, testspec_path: Path) -> None:
 
         group["valid"] = vm
 
-    try:
-        with testspec_path.open("w", encoding="utf-8") as fh:
-            yaml_dumper.dump(doc, fh)
-    except Exception as e:
-        raise TedsError(
-            f"Failed to write testspec: {testspec_path}\n  error: {type(e).__name__}: {e}"
-        ) from e
+    logger.debug(f"generate_exact_node: About to write file {testspec_path}")
+    _write_yaml(testspec_path, doc)
 
 
 def generate_from(parent_ref: str, testspec_path: Path) -> None:
@@ -93,9 +123,10 @@ def generate_from(parent_ref: str, testspec_path: Path) -> None:
         tests = {}
         doc["tests"] = tests
 
-    base_dir = testspec_path.parent
+    base_dir = Path.cwd()  # Schema files are always relative to working directory
 
     # Try to resolve the schema reference
+    logger.debug(f"generate_exact_node: base_dir={base_dir}, parent_ref={parent_ref}, testspec_path={testspec_path}")
     try:
         parent_node, parent_frag = resolve_schema_node(base_dir, parent_ref)
     except Exception as first_error:
@@ -123,13 +154,7 @@ def generate_from(parent_ref: str, testspec_path: Path) -> None:
             ) from first_error
     file_part, _, _ = parent_ref.partition("#")
     if not isinstance(parent_node, dict):
-        try:
-            with testspec_path.open("w", encoding="utf-8") as fh:
-                yaml_dumper.dump(doc, fh)
-        except Exception as e:
-            raise TedsError(
-                f"Failed to write testspec: {testspec_path}\n  error: {type(e).__name__}: {e}"
-            ) from e
+        _write_yaml(testspec_path, doc)
         return
 
     # Create tests for children of the referenced node (standard behavior)
@@ -153,13 +178,7 @@ def generate_from(parent_ref: str, testspec_path: Path) -> None:
 
             group["valid"] = vm
 
-    try:
-        with testspec_path.open("w", encoding="utf-8") as fh:
-            yaml_dumper.dump(doc, fh)
-    except Exception as e:
-        raise TedsError(
-            f"Failed to write testspec: {testspec_path}\n  error: {type(e).__name__}: {e}"
-        ) from e
+    _write_yaml(testspec_path, doc)
 
 
 def parse_generate_config(config_str: str) -> dict[str, dict[str, Any]] | str:
@@ -169,6 +188,7 @@ def parse_generate_config(config_str: str) -> dict[str, dict[str, Any]] | str:
         - dict: Source-centric configuration {source_file: config}
         - str: JSON Pointer string for backward compatibility
     """
+    logger.debug(f"parse_generate_config: input config_str='{config_str}'")
     # Handle @filename syntax
     if config_str.startswith("@"):
         config_file = Path(config_str[1:])
@@ -176,6 +196,7 @@ def parse_generate_config(config_str: str) -> dict[str, dict[str, Any]] | str:
             raise TedsError(f"Configuration file not found: {config_file}")
         try:
             config_str = config_file.read_text(encoding="utf-8")
+            logger.debug(f"parse_generate_config: loaded config file content: {repr(config_str)}")
         except Exception as e:
             raise TedsError(
                 f"Failed to read configuration file {config_file}: {e}"
@@ -184,6 +205,7 @@ def parse_generate_config(config_str: str) -> dict[str, dict[str, Any]] | str:
     # Parse YAML
     try:
         parsed = yaml_loader.load(config_str)
+        logger.debug(f"parse_generate_config: parsed YAML: {parsed}")
     except Exception as e:
         raise TedsError(f"Invalid YAML configuration: {e}") from e
 
@@ -222,56 +244,60 @@ def parse_generate_config(config_str: str) -> dict[str, dict[str, Any]] | str:
                         f"Source '{source_file}': path must be string, got {type(path).__name__}"
                     )
 
+        logger.debug(f"parse_generate_config: returning normalized_config={normalized_config}")
         return normalized_config
     elif isinstance(parsed, str):
-        # Backward compatibility: JSON Pointer string
-        return parsed
+        # Check for REF[=TARGET] syntax first
+        if "=" in parsed:
+            ref_part, target_part = parsed.split("=", 1)
+        else:
+            ref_part = parsed
+            target_part = None
+
+        # JSON Pointer string - convert to normalized JSON-Path configuration
+        if "#" in ref_part:
+            file_part, fragment = ref_part.split("#", 1)
+            if not file_part:
+                raise TedsError(f"Missing schema file path: {ref_part}")
+
+            # Convert JSON Pointer to JSON-Path with mandatory wildcard for children
+            fragment = fragment.lstrip("/")
+            if fragment:
+                # Convert /path/to/property to $["path"]["to"]["property"].*
+                parts = fragment.split("/")
+                quoted_parts = [f'["{part}"]' for part in parts]
+                jsonpath = "$" + "".join(quoted_parts) + ".*"
+            else:
+                # Root reference becomes $.*
+                jsonpath = "$.*"
+
+            # Expand template variables in target if present
+            expanded_target = None
+            if target_part:
+                expanded_target = expand_target_template(
+                    target_part, file_part, fragment
+                )
+
+            # Return normalized configuration with expanded target
+            return {file_part: {"paths": [jsonpath], "target": expanded_target}}
+        else:
+            # String without fragment - RFC 3986 compliant: references whole document
+            # Both "schema.yaml" and "schema.yaml#" reference the entire document
+            jsonpath = "$.*"
+
+            # Expand template variables in target if present
+            expanded_target = None
+            if target_part:
+                expanded_target = expand_target_template(target_part, ref_part, "")
+
+            return {ref_part: {"paths": [jsonpath], "target": expanded_target}}
     else:
         raise TedsError(
             f"Configuration must be object or string, got {type(parsed).__name__}"
         )
 
 
-def validate_jsonpath_expression(expr: str) -> None:
-    """Validate a JsonPath expression for basic syntax."""
-    # Handle backward compatibility: JSON Pointer format (with #)
-    if "#" in expr:
-        file_part, fragment = expr.split("#", 1)
-        if not file_part:
-            raise TedsError(f"Missing schema file path: {expr}")
-
-        # Convert to JsonPath for validation
-        if fragment.startswith("/"):
-            fragment = fragment[1:]  # Remove leading slash
-        if not fragment:
-            return  # Root reference is valid
-
-        # Check for basic syntax errors in JSON Pointer format
-        invalid_patterns = [
-            r"\.\./",  # Parent navigation (not supported)
-        ]
-        for pattern in invalid_patterns:
-            if re.search(pattern, fragment):
-                raise TedsError(f"Invalid JSON Pointer expression: {expr}")
-        return
-
-    # Pure JsonPath expression validation
-    if not expr:
-        raise TedsError("Empty JsonPath expression")
-
-    # Basic JsonPath validation - must start with $ for root
-    if not expr.startswith("$"):
-        raise TedsError(f"JsonPath expression must start with '$': {expr}")
-
-    # Check for basic syntax errors
-    invalid_patterns = [
-        r"\[([^]]+$)",  # Unclosed bracket
-        r"\.\./",  # Parent navigation (not supported)
-    ]
-
-    for pattern in invalid_patterns:
-        if re.search(pattern, expr):
-            raise TedsError(f"Invalid JsonPath expression: {expr}")
+# validate_jsonpath_expression() function removed - redundant to jsonpath-ng validation
 
 
 def _find_schema_file(base_dir: Path, filename: str) -> Path | None:
@@ -293,104 +319,42 @@ def _find_schema_file(base_dir: Path, filename: str) -> Path | None:
     return None
 
 
-def expand_jsonpath_expressions(source_file: Path, expressions: list[str]) -> list[str]:
+def expand_jsonpath_expressions(source_file: Path, expressions: list[str], reference_path: str = None) -> list[str]:
     """Expand JsonPath expressions to concrete JSON Pointer references.
 
-    This function follows the JSONPath specification exactly - no interpretation,
-    no "intelligent" fallback logic. jsonpath-ng does the work correctly.
+    This function processes only JSON-Path expressions. JSON-Pointer conversion
+    should happen at the CLI boundary via parse_generate_config().
     """
     expanded = []
 
     # Load schema document once
+    logger.debug(f"expand_jsonpath_expressions: source_file={source_file}")
     try:
         schema_doc = yaml_loader.load(source_file.read_text(encoding="utf-8")) or {}
     except Exception as e:
         raise TedsError(f"Failed to load schema {source_file}: {e}") from e
 
-    for expr in expressions:
-        validate_jsonpath_expression(expr)
-
-        # Handle backward compatibility: JSON Pointer format
-        if "#" in expr:
-            file_part, fragment = expr.split("#", 1)
-            if "*" not in fragment:
-                # No wildcards - append wildcard for backward compatibility (children expansion)
-                expr = f"{file_part}#{fragment}/*" if fragment else f"{file_part}#/*"
-
-            # Convert JSON Pointer to JsonPath format for expansion
-            jsonpath_expr = fragment.lstrip("/")
-            if jsonpath_expr:
-                # Convert /path/to/property to $["path"]["to"]["property"]
-                parts = jsonpath_expr.split("/")
-                quoted_parts = [f'["{part}"]' for part in parts]
-                jsonpath_expr = "$" + "".join(quoted_parts)
-            else:
-                jsonpath_expr = "$"
-        else:
-            # Pure JsonPath expression - let jsonpath-ng handle it exactly
-            jsonpath_expr = expr
-
+    for jsonpath_expr in expressions:
         try:
-            # Parse and find matches - jsonpath-ng knows what it's doing
+            # Parse and find matches - jsonpath-ng handles all validation
             jsonpath_parser = jsonpath_ng.parse(jsonpath_expr)
             matches = jsonpath_parser.find(schema_doc)
 
             for match in matches:
-                # Convert path back to JSON Pointer
-                path_parts = _convert_jsonpath_match_to_json_pointer_parts(
-                    match.full_path
-                )
-
-                if path_parts:
-                    json_pointer = "/" + "/".join(path_parts)
-                    expanded.append(f"{source_file.name}#{json_pointer}")
-                else:
-                    expanded.append(f"{source_file.name}#/")
+                # Use the clean utils function for JSONPath -> JSON Pointer conversion
+                json_pointer = json_path_to_json_pointer(match.full_path)
+                # Use reference_path if provided (for correct relative paths in test files)
+                ref_name = reference_path if reference_path else source_file.name
+                expanded.append(f"{ref_name}#{json_pointer}")
 
         except Exception as e:
             raise TedsError(
-                f"Failed to expand JsonPath expression '{expr}': {e}"
+                f"Failed to expand JsonPath expression '{jsonpath_expr}': {e}"
             ) from e
 
     return expanded
 
 
-def _convert_jsonpath_match_to_json_pointer_parts(full_path: str) -> list[str]:
-    """Convert jsonpath-ng full_path to JSON Pointer parts.
-
-    jsonpath-ng returns paths in formats like:
-    - '$defs'.User -> ['$defs', 'User']
-    - items.[0] -> ['items', '0']  (convert [0] to 0)
-    - key.nested.prop -> ['key', 'nested', 'prop']
-    """
-    path_str = str(full_path)
-    if path_str.startswith("$"):
-        path_str = path_str[1:]  # Remove $
-
-    if not path_str:
-        return []
-
-    path_parts = []
-
-    # Handle dot notation parsing
-    if path_str.startswith("."):
-        path_str = path_str[1:]  # Remove leading .
-
-    for part in path_str.split("."):
-        # Handle quoted parts like '$defs' -> $defs
-        if part.startswith("'") and part.endswith("'"):
-            part = part[1:-1]
-        # Convert array bracket notation [0] -> 0
-        elif part.startswith("[") and part.endswith("]"):
-            # Remove brackets and quotes if present
-            part = part[1:-1]
-            if part.startswith("'") and part.endswith("'"):
-                part = part[1:-1]
-        path_parts.append(part)
-
-    return path_parts
-
-
 def detect_conflicts(expanded_refs: list[str]) -> list[tuple[str, list[int]]]:
     """Detect conflicts between multiple source references.
 
@@ -424,22 +388,42 @@ def generate_from_source_config(
     config: dict[str, dict[str, Any]], base_dir: Path
 ) -> None:
     """Generate test specifications from source-centric YAML configuration."""
+    logger.debug(f"generate_from_source_config: base_dir={base_dir}, config={config}")
 
     for source_file_str, source_config in config.items():
-        source_file = base_dir / source_file_str
+        logger.debug(f"Processing source: source_file_str='{source_file_str}', base_dir={base_dir}")
 
-        # Determine target file
+        # Determine target file (no schema path resolution yet)
         if source_config["target"]:
-            # Explicit target: relative to cwd (standard CLI behavior)
-            target_name = source_config["target"].replace("{base}", source_file.stem)
+            # Explicit target: relative to cwd with template variable expansion
+            target_name = expand_target_template(source_config["target"], source_file_str, "")
             target_path = base_dir / target_name
+            logger.debug(f"Explicit target: target_name='{target_name}', target_path={target_path}")
         else:
             # Default: {base}.tests.yaml next to schema file
-            target_name = f"{source_file.stem}.tests.yaml"
-            target_path = source_file.parent / target_name
+            schema_file = base_dir / source_file_str
+            target_name = f"{schema_file.stem}.tests.yaml"
+            target_path = schema_file.parent / target_name  
+            logger.debug(f"Default target: target_name='{target_name}', target_path={target_path}")
 
-        # Expand JsonPath expressions for this source
-        expanded_refs = expand_jsonpath_expressions(source_file, source_config["paths"])
+        # Determine schema file path for JsonPath expansion
+        if source_config["target"]:
+            # For explicit targets, schema paths are relative to target directory
+            schema_file = target_path.parent / source_file_str
+        else:
+            # For default targets, schema paths are relative to working directory
+            schema_file = base_dir / source_file_str
+        
+        # Calculate correct reference path relative to test file directory
+        try:
+            reference_path = os.path.relpath(schema_file, target_path.parent)
+        except ValueError:
+            # Fallback if relative path calculation fails
+            reference_path = source_file_str
+        
+        logger.debug(f"Before expand_jsonpath_expressions: schema_file={schema_file}, paths={source_config['paths']}, reference_path={reference_path}")
+        expanded_refs = expand_jsonpath_expressions(schema_file, source_config["paths"], reference_path)
+        logger.debug(f"After expand_jsonpath_expressions: expanded_refs={expanded_refs}")
 
         # Detect and warn about conflicts within this source
         conflicts = detect_conflicts(expanded_refs)
@@ -459,10 +443,30 @@ def generate_from_source_config(
                 unique_refs.append(ref)
 
         # Generate tests for each unique reference (exact nodes only, no children expansion)
-        if unique_refs:  # Only print if we're actually generating something
-            print(f"Generating {target_path}", file=sys.stderr)
-        for ref in unique_refs:
-            try:
-                generate_exact_node(ref, target_path)
-            except Exception as e:
-                print(f"Warning: Failed to generate from '{ref}': {e}", file=sys.stderr)
+        try:
+            display_path = str(target_path.relative_to(Path.cwd()))
+        except ValueError:
+            # Fall back to absolute path if relative conversion fails (e.g., symlinks)
+            display_path = str(target_path)
+        print(f"Generating {display_path}", file=sys.stderr)
+
+        if unique_refs:
+            # Generate for each found reference
+            for ref in unique_refs:
+                try:
+                    # For schema resolution in generate_exact_node, always use test file directory
+                    # because references are now calculated relative to test file
+                    schema_base_dir = target_path.parent
+                    
+                    logger.debug(f"Calling generate_exact_node with ref='{ref}', target_path={target_path}, schema_base_dir={schema_base_dir}")
+                    generate_exact_node(ref, target_path, schema_base_dir=schema_base_dir)
+                    logger.debug(f"Successfully generated from ref='{ref}'")
+                except Exception as e:
+                    logger.debug(f"Exception in generate_exact_node: {type(e).__name__}: {e}")
+                    print(
+                        f"Warning: Failed to generate from '{ref}': {e}",
+                        file=sys.stderr,
+                    )
+        else:
+            # No references found - create empty test file for visibility and batch-mode feedback
+            _create_empty_test_file(target_path)
