@@ -1,6 +1,8 @@
 """BDD tests for TeDS generate command - reorganized from original working files."""
 
+import logging
 import os
+import re
 import subprocess
 import sys
 import tempfile
@@ -12,6 +14,10 @@ from pytest_bdd import given, parsers, scenarios, then, when
 from teds_core.cli import main as cli_main
 from teds_core.yamlio import yaml_loader
 
+# Set up logging for test debugging
+logging.basicConfig(level=logging.DEBUG, format='TEST:%(levelname)s: %(message)s', stream=sys.stderr)
+test_logger = logging.getLogger('test_bdd')
+
 
 def run_teds_command(*args):
     """Helper function to run teds CLI commands in tests."""
@@ -77,6 +83,7 @@ def working_directory(temp_workspace):
 def create_subdirectory(temp_workspace, dirname):
     """Create a subdirectory."""
     subdir = temp_workspace / dirname
+    test_logger.debug(f"Creating subdirectory: {dirname} at {subdir}")
     subdir.mkdir(parents=True, exist_ok=True)
 
 
@@ -84,6 +91,7 @@ def create_subdirectory(temp_workspace, dirname):
 def create_schema_file(temp_workspace, schema_files, filename, docstring):
     """Create a schema file with specified content."""
     schema_path = temp_workspace / filename
+    test_logger.debug(f"Creating schema file: {filename} at {schema_path}")
     schema_path.parent.mkdir(parents=True, exist_ok=True)
 
     # Remove yaml language marker if present
@@ -91,7 +99,10 @@ def create_schema_file(temp_workspace, schema_files, filename, docstring):
     if content and content.strip().startswith("yaml"):
         content = "\n".join(content.strip().split("\n")[1:])
 
-    schema_path.write_text(content.strip() if content else "", encoding="utf-8")
+    final_content = content.strip() if content else ""
+    test_logger.debug(f"Schema file {filename} content: {repr(final_content)}")
+    schema_path.write_text(final_content, encoding="utf-8")
+    test_logger.debug(f"Schema file written to: {schema_path}")
     schema_files[filename] = schema_path
 
 
@@ -105,7 +116,10 @@ def create_config_file(temp_workspace, config_files, filename, docstring):
     if content.strip().startswith("yaml"):
         content = "\n".join(content.strip().split("\n")[1:])
 
-    config_path.write_text(content.strip(), encoding="utf-8")
+    final_content = content.strip()
+    test_logger.debug(f"Creating config file {filename} with content: {repr(final_content)}")
+    config_path.write_text(final_content, encoding="utf-8")
+    test_logger.debug(f"Config file created at: {config_path}")
     config_files[filename] = config_path
 
 
@@ -126,6 +140,7 @@ def create_existing_test_file(temp_workspace, filename, docstring):
 @when(parsers.parse("I run the generate command: `{command}`"))
 def run_generate_command(temp_workspace, cli_result, command):
     """Execute the generate command."""
+    test_logger.debug(f"Running command: {command} in workspace: {temp_workspace}")
     # Extract command arguments
     # Remove 'teds generate ' prefix
     args_str = command.replace("teds generate ", "").strip()
@@ -148,6 +163,17 @@ def run_generate_command(temp_workspace, cli_result, command):
     cli_result["returncode"] = result.returncode
     cli_result["stdout"] = result.stdout
     cli_result["stderr"] = result.stderr
+    
+    # Debug output for comparison between working and failing tests
+    test_logger.debug(f"Command result: returncode={result.returncode}")
+    test_logger.debug(f"Stderr: {result.stderr}")
+    test_logger.debug(f"Files created: {[f.name for f in temp_workspace.iterdir() if f.is_file()]}")
+    print(f"DEBUG: Command: {command}")
+    print(f"DEBUG: Args: {args}")
+    print(f"DEBUG: Return code: {result.returncode}")
+    print(f"DEBUG: Stderr: {result.stderr}")
+    print(f"DEBUG: Working directory: {temp_workspace}")
+    print(f"DEBUG: Files created: {[f.name for f in temp_workspace.iterdir() if f.is_file()]}")
 
     # Store result for later assertions
     pytest.current_exit_code = result.returncode
@@ -212,7 +238,7 @@ def verify_test_file_exists(temp_workspace, test_files, filename):
     test_files_found = [
         f for f in actual_files if f.endswith(".tests.yaml") or f.endswith(".yaml")
     ]
-
+    
     assert test_path.exists(), (
         f"Test file '{filename}' was not created.\n"
         f"Expected: {filename}\n"
@@ -223,17 +249,13 @@ def verify_test_file_exists(temp_workspace, test_files, filename):
 
 
 @then(parsers.parse('the test file should contain "{content}"'))
-def step_test_file_should_contain(temp_workspace, content):
+def step_test_file_should_contain(temp_workspace, test_files, content):
     """Assert that the most recently created test file contains specific content."""
-    # Find the most recent .tests.yaml file
-    test_files = list(temp_workspace.glob("*.tests.yaml"))
-    if not test_files:
-        test_files = list(temp_workspace.glob("*.yaml"))
-
-    assert test_files, "No test files found"
-
-    # Use the most recently created file
-    latest_file = max(test_files, key=lambda f: f.stat().st_mtime)
+    # Use the test files from context (stored by previous steps)
+    assert test_files, "No test files found in context"
+    
+    # Use the most recently added file from context
+    latest_file = max(test_files.values(), key=lambda f: f.stat().st_mtime)
     file_content = latest_file.read_text()
     assert (
         content in file_content
@@ -357,76 +379,77 @@ def command_should_succeed(cli_result):
     ), f"Command failed with exit code {cli_result['returncode']}. Stderr: {cli_result['stderr']}"
 
 
-@then(parsers.parse('the error output should contain "{expected_text}"'))
+@then(parsers.parse('the error output should match "{expected_text}"'))
 def verify_error_output(cli_result, expected_text):
     """Verify that the error output contains the expected text."""
     stderr = cli_result["stderr"] or ""
-    assert (
-        expected_text in stderr
+    assert re.fullmatch(
+        expected_text, stderr, re.DOTALL
     ), f"Expected '{expected_text}' in stderr, but got: {stderr}"
 
 
-@then(
-    parsers.parse('a test file "{filename}" should be created with content:'),
-    target_fixture="verified_test_file",
-)
-def verify_test_file_created(temp_workspace, test_files, filename):
+@then(parsers.parse('a test file "{filename}" should be created with content:'))
+def verify_test_file_created_with_content(
+    temp_workspace, test_files, filename, docstring
+):
     """Verify that a test file was created with expected content."""
+    expected_content = docstring
+    test_path = temp_workspace / filename
 
-    def _verify_with_content(expected_content):
-        test_path = temp_workspace / filename
-        assert test_path.exists(), f"Test file {filename} was not created"
+    # Always check file existence first
+    if not test_path.exists():
+        available_files = [f.name for f in temp_workspace.iterdir()]
+        raise AssertionError(
+            f"Test file {filename} was not created!\n"
+            f"Expected file: {test_path}\n"
+            f"Available files: {available_files}"
+        )
 
-        # Read actual content
-        actual_content = test_path.read_text(encoding="utf-8")
+    # Read actual content
+    actual_content = test_path.read_text(encoding="utf-8")
 
-        # Remove yaml language marker if present
-        if expected_content.strip().startswith("yaml"):
-            expected_content = "\n".join(expected_content.strip().split("\n")[1:])
-
-        # Parse both as YAML for comparison
+    # Parse both as YAML for comparison (pytest-bdd already removes language markers)
+    try:
         actual_yaml = yaml_loader.load(actual_content)
-        expected_yaml = yaml_loader.load(expected_content.strip())
+        expected_yaml = yaml_loader.load(expected_content)
+    except Exception as e:
+        raise AssertionError(
+            f"YAML parsing error: {e}\n"
+            f"Expected content:\n{expected_content}\n"
+            f"Actual content:\n{actual_content}"
+        ) from e
 
-        assert (
-            actual_yaml == expected_yaml
-        ), f"Test file content mismatch.\nExpected:\n{expected_content}\nActual:\n{actual_content}"
+    if actual_yaml != expected_yaml:
+        raise AssertionError(
+            f"Test file content mismatch!\n"
+            f"Expected:\n{expected_content}\n"
+            f"Actual:\n{actual_content}"
+        )
 
-        # Store for further verification
-        test_files[filename] = test_path
-        return test_path
-
-    return _verify_with_content
+    # Store for further verification
+    test_files[filename] = test_path
 
 
 @then(parsers.parse('the test file "{filename}" should be updated with content:'))
-def verify_test_file_updated(temp_workspace, test_files, filename):
+def verify_test_file_updated(temp_workspace, test_files, filename, docstring):
     """Verify that a test file was updated with expected content."""
+    expected_content = docstring
+    test_path = temp_workspace / filename
+    assert test_path.exists(), f"Test file {filename} does not exist"
 
-    def _verify_updated_content(expected_content):
-        test_path = temp_workspace / filename
-        assert test_path.exists(), f"Test file {filename} does not exist"
+    # Read actual content
+    actual_content = test_path.read_text(encoding="utf-8")
 
-        # Read actual content
-        actual_content = test_path.read_text(encoding="utf-8")
+    # Parse both as YAML for comparison (pytest-bdd already removes language markers)
+    actual_yaml = yaml_loader.load(actual_content)
+    expected_yaml = yaml_loader.load(expected_content)
 
-        # Remove yaml language marker if present
-        if expected_content.strip().startswith("yaml"):
-            expected_content = "\n".join(expected_content.strip().split("\n")[1:])
+    assert (
+        actual_yaml == expected_yaml
+    ), f"Test file content mismatch.\nExpected:\n{expected_content}\nActual:\n{actual_content}"
 
-        # Parse both as YAML for comparison
-        actual_yaml = yaml_loader.load(actual_content)
-        expected_yaml = yaml_loader.load(expected_content.strip())
-
-        assert (
-            actual_yaml == expected_yaml
-        ), f"Test file content mismatch.\nExpected:\n{expected_content}\nActual:\n{actual_content}"
-
-        # Store for further verification
-        test_files[filename] = test_path
-        return test_path
-
-    return _verify_updated_content
+    # Store for further verification
+    test_files[filename] = test_path
 
 
 @then(parsers.parse('the result should not contain "{unwanted_ref}"'))
